import os
from contextlib import contextmanager
from shutil import rmtree
from tempfile import mkdtemp

DATA_DIR = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'data')
SAMPLE_PAGE_FILE_PATH = os.path.join(DATA_DIR, 'page.md')


@contextmanager
def temporary_folder():
    """
    Creates a temporary folder, return it and delete it afterwards.
    This allows to do something like this in tests:
        >>> with temporary_folder() as d:
            # do whatever you want
    """
    tempdir = mkdtemp()
    try:
        yield tempdir
    finally:
        rmtree(tempdir)



from tests import SAMPLE_PAGE_FILE_PATH
from tir.posts import Post


def test_post_reader():
    p = Post(SAMPLE_PAGE_FILE_PATH)
    assert isinstance(p, Post)
    assert p.meta['title'] == 'Sample page'



import hashlib
import os
import platform
import sys
import warnings
from datetime import datetime
from os import listdir
from os.path import join
from urllib.parse import urlparse

import yaml


def app_config():
    try:
        with open('tir.yml', encoding='utf-8') as f:
            config = yaml.safe_load(f.read().encode('utf-8'))
            return config
    except FileNotFoundError:
        warnings.warn('No configuration file was found.')


def remove_list_meta(meta):
    """ Extract first and single element of list from meta dic
    :rtype: dict
    """
    if isinstance(meta, dict):
        for mk, mv in meta.items():
            if isinstance(mv, list) and len(mv) == 1:
                meta[mk] = mv[0]
    return meta


def is_windows():
    return sys.platform.startswith("win") or (sys.platform == "cli" and os.name == "nt")


def is_linux():
    return platform.system() == 'Linux'


def scan_dir(directory=None, extensions=None):
    """
    Scan a directory for files with a specific or list of extensions then return matching results

    :param directory: absolute path to directory to scan
    :type directory: str
    :param extensions: extensions to search for
    :type extensions: list
    """
    matches = []
    if extensions is not None:
        extensions = ['.' + ext for ext in extensions]
    files = listdir(directory)
    for file in files:
        if extensions is not None:
            for extension in extensions:
                if file.endswith(extension):
                    matches.append(join(directory, file))
        else:
            matches.append(join(directory, file))
    return matches


def mktree(path):
    if not os.path.exists(path):
        os.makedirs(path)


def _(text):
    translations = {
        'created': 'créé',
        'updated': 'mis à jour',
        'modified': 'modifié',
        'status': 'statut',
        'confidence': 'confiance',
        'difficulty': 'difficulté'
    }
    if text in translations:
        return translations[text]
    return text


def url_for(route, slug=None, filename=''):
    if route == 'index':
        return '/'
    if route == 'post':
        return slug + '.html'
    elif route == 'static':
        return '{}/{}'.format('static', filename)


def format_date(value):
    try:
        d = datetime.strptime(value, '%Y/%m/%d')
        return d
    except (ValueError, TypeError):
        return value


def n_bit_hash(string: str, n: int) -> int:
    return int(hashlib.blake2b(string.encode('utf-8')).hexdigest(), 16) % 10 ** n


def get_domain(url: str, include_ext=True) -> str:
    domain = urlparse(url).netloc
    domain_l = domain.split('.')
    if len(domain_l) >= 2:
        domain = domain_l[-2]
    return domain


def scantree(path):
    """Recursively yield DirEntry objects for given directory."""
    for entry in os.scandir(path):
        if entry.is_dir(follow_symlinks=False):
            yield from scantree(entry.path)  # see below for Python 2.x
        else:
            yield entry



import logging
import os
import shutil
import time

import sass

from tir.config import REQUIRED_PATHS, get_config
from tir.files import hash_content, rm_dir_files
from tir.posts import Post
from tir.templates import TemplateLoader
from tir.tools import is_init, minify_file
from tir.utils import mktree, scantree

logger = logging.getLogger(__name__)


class Tir(object):

    def __init__(self):
        """Tir def

        Performs some checks on the environment before doing anything else.
        """

        self.conf = get_config()

        self.theme = self.conf.visuals.theme
        self.lang = self.conf.lang
        self.build_dir = self.conf.build_dir
        self.working_dir = os.getcwd()
        self.content_dir = os.path.join(self.working_dir, 'content')
        self.posts_dir = os.path.join(self.content_dir, 'posts')
        self.notes_dir = os.path.join(self.content_dir, 'notes')

        self.oc = []

    @staticmethod
    def init():
        if is_init():
            return False
        print('Initializing Tir project...')
        skeleton_dirs = REQUIRED_PATHS
        skeleton_files = ['tir.yml']
        for skeleton_dir in skeleton_dirs:
            mktree(skeleton_dir)
        for skeleton_file in skeleton_files:
            shutil.copyfile(
                pkg_resources.resource_filename(
                    'tir',
                    'data/{}'.format(skeleton_file)
                ), '{}/{}'.format(os.getcwd(), skeleton_file)
            )

        # copy quickstart data
        shutil.copytree(
            pkg_resources.resource_filename('tir', 'data'),
            os.path.join(os.getcwd()),
            dirs_exist_ok=True
        )
        print('Tir project was successfully installed.')
        return True

    def build(self, path: str = None):
        if not is_init():
            print(
                'Project does not seem to be initialized. Type "tir init" to create a Tir project here.')
            return False
        try:
            if os.path.exists(self.build_dir) and not path:
                shutil.rmtree(self.build_dir)
            tpl_visuals_dir = os.path.join(os.getcwd(), 'layout', self.theme)
            print('Selected template:', tpl_visuals_dir)
            print('Replace assets files')
            assets_src_dir = os.path.join(tpl_visuals_dir, 'assets')
            assets_target_dir = os.path.join(
                os.getcwd(), self.build_dir, 'static')

            tpl_loader = TemplateLoader(
                layout_directory=tpl_visuals_dir, config=self.conf)

            print('Building static files...')
            css_dir = os.path.join(assets_target_dir, 'css')
            mktree(css_dir)
            compiled_scss = sass.compile(filename=os.path.join(
                assets_src_dir, 'scss', 'main.scss'))
            stylesheet_path = os.path.join(css_dir, 'stylesheet.css')
            with open(stylesheet_path, 'w+', encoding='utf-8') as f:
                f.write(compiled_scss)
            minified_stylesheet_path = minify_file(stylesheet_path)

            shutil.copytree(
                os.path.join(tpl_visuals_dir, 'assets', 'images'),
                os.path.join(self.build_dir, 'static', 'images')
            )

            elements = []

            print('Building content...')

            for entry in scantree(self.posts_dir):
                if entry.is_file():
                    p = Post(entry.path, css_path=minified_stylesheet_path, tpl_loader=tpl_loader)
                    elements.append(p)

            for entry in elements:
                entry.write()
            print('Build was successful.')
            return True

        except Exception as e:
            raise e

    def watch(self):
        self.oc = hash_content(self.posts_dir)
        print('Now watching for changes...'.format(self.posts_dir))
        while 1:
            time.sleep(2)
            cc = hash_content(self.posts_dir)
            if self.oc == cc:
                pass
            else:
                self.oc = cc
                self.build()



import logging
from pathlib import Path

import marshmallow_dataclass
import yaml

from tir.schemas.config import Config

logger = logging.getLogger(__name__)

REQUIRED_PATHS = ['tir.yml', 'content', 'content/posts', 'layout']
_DEFAULT_CONF_FILE = 'tir.yml'
config_schema = marshmallow_dataclass.class_schema(Config)()


def get_config(path: str = None) -> Config:
    config_file_path: Path = Path.cwd().joinpath(path or _DEFAULT_CONF_FILE)
    if config_file_path.exists():
        with open(config_file_path, 'r', encoding='utf-8') as f:
            content = f.read()
            cfg = yaml.safe_load(content)
            config: Config = config_schema.load(cfg)

    else:
        config: Config = config_schema.load({})
    return config



import pathlib

import marshmallow_dataclass
import yaml
from tir.config import get_config
from tir.schemas.config import Config

from tests import SAMPLE_PAGE_FILE_PATH


def test_config_obj():

    with open(pathlib.Path(__file__).parent.joinpath('data', 'tir.yml')) as f:
        cfg = yaml.safe_load(f.read())
        deployment_schema = marshmallow_dataclass.class_schema(Config)()
        deployment = deployment_schema.load(cfg)
        assert deployment is not None


def test_config_singleton_like():
    n: Config = get_config()
    p: Config = get_config()
    assert n == p


def test_load_config():
    config = get_config()
    assert config.build_dir == 'html'



import os
import pathlib
from os import getenv
from os.path import join, normpath

import markdown
from markdown.extensions.wikilinks import WikiLinkExtension

from tir import get_config
from tir.parsers.markdown.links import CustomInlineLinksExtension
from tir.parsers.markdown.local_links import LocalLinksExtension
from tir.utils import remove_list_meta

_EXCLUDED_FILES = ['index.md']


class Post:
    BASE_DIR = getenv('BASE_DIR', os.getcwd())
    CONTENT_DIR = normpath(join(BASE_DIR, 'content'))
    POSTS_DIR = join(CONTENT_DIR, 'posts')
    RETROS_DIR = normpath(join(CONTENT_DIR, 'retrospectives'))
    LINKS_DIR = normpath(join(CONTENT_DIR, 'links/'))
    MISC_DIR = normpath(join(CONTENT_DIR, 'misc/'))

    def __init__(self, source: str = None, css_path=None, tpl_loader=None, online_only=True, lang='en'):
        self.source: str = source
        self.css_path = css_path
        self.tpl_loader = tpl_loader
        self.target_path = None
        self.file_base_name = os.path.basename(os.path.splitext(self.source)[0])
        self.meta = None
        self.raw = None
        self.content = None
        self.conf = get_config()
        self.online_only = online_only
        self.lang = lang
        self.parse()

    def parse(self):
        try:
            with open(self.source, 'r', encoding='utf8') as f:
                self.raw = f.read()
            if not self.raw:
                return
            md = markdown.Markdown(
                extensions=[
                    'markdown.extensions.meta',
                    'markdown.extensions.toc',
                    'markdown.extensions.footnotes',
                    'markdown.extensions.def_list',
                    'markdown.extensions.tables',
                    'markdown.extensions.sane_lists',
                    WikiLinkExtension(
                        base_url=f'https://{self.lang}.wikipedia.org/wiki/', end_url=''),
                    LocalLinksExtension(),
                    CustomInlineLinksExtension()
                ]
            )

            self.content = md.convert(self.raw)
            if hasattr(md, 'Meta') and md.Meta:
                meta = md.Meta
                meta = remove_list_meta(meta)
                if 'online' in meta and (meta['online'] == 'false' or self.online_only):
                    pass
                if hasattr(md, 'toc'):
                    meta['contents'] = md.toc
                self.meta = meta
        except FileNotFoundError as fnf:
            raise fnf

    def __repr__(self):
        return f'<Post path={self.source}, file_name_base={self.file_base_name} />'

    def write(self):
        target_path = pathlib.Path(self.conf.build_dir) / pathlib.Path(self.source).parent.relative_to(self.POSTS_DIR) / (self.meta['slug'] + self.conf.file_extension)
        print('target full path: %s' % target_path)
        os.makedirs(pathlib.Path(target_path).parent, exist_ok=True)
        with open(target_path, 'w', encoding='utf-8') as fh:
            head = {
                'stylesheet_file_name': self.css_path
            }
            fh.write(self.tpl_loader.env.get_template(
                'index.html' if self.file_base_name == 'index' else 'post.html').render(
                post=self,
                head=head,
                recent_posts=[3]
            ))



from tests import temporary_folder
from tir.utils import n_bit_hash


def test_simple():
    with temporary_folder() as tf:
        assert tf is not None, 'Folder object is not None'


def test_n_bit_hash(load_dummy_file_content):
    for i in range(1, 256):
        h = n_bit_hash(load_dummy_file_content, i)
        assert isinstance(h, int)
        assert h > 0



from dataclasses import field
from datetime import datetime
from pathlib import Path, PosixPath, WindowsPath
from typing import ClassVar, List, Optional, Type, Union, Any

from marshmallow import Schema, fields
from marshmallow_dataclass import dataclass


@dataclass
class Deployment(Schema):
    class Meta:
        ordered: True

    host: str
    port: int
    path: str
    username: str
    password: Optional[str]
    key_file: Optional[str] = field(metadata=dict(data_key='key-file'))
    Schema: ClassVar[Type[Schema]] = Schema


@dataclass
class ConfigMeta(Schema):
    class Meta:
        ordered: True

    description: str
    title: str
    copyright: Optional[str]
    Schema: ClassVar[Type[Schema]] = Schema


@dataclass
class NavigationItem(Schema):
    class Meta:
        ordered: True

    link: str
    title: str
    Schema: ClassVar[Type[Schema]] = Schema


@dataclass
class Visuals(Schema):
    class Meta:
        ordered: True

    theme: str = field(default="default")
    Schema: ClassVar[Type[Schema]] = Schema


@dataclass
class Config(Schema):
    class Meta:
        ordered: True

    build_dir: str
    deployment: Optional[Deployment]
    meta: ConfigMeta = field(default_factory=dict)
    navigation: List[NavigationItem] = field(default_factory=list)
    recent_post_counter: int = field(
        metadata=dict(data_key='recent-post-counter'), default=5)
    visuals: Visuals = field(default_factory=Visuals)
    file_extension: str = field(
        default='html', metadata=(dict(missing=False, allow_none=True)))
    lang: str = field(default='en_US')
    working_dir: Any = field(default=Path.cwd())
    Schema: ClassVar[Type[Schema]] = Schema

    def __post_init__(self):
        self.file_extension = "" if self.file_extension is None else f'.{self.file_extension}'



"""
todos:
* create html directory if doesn't exist
* format date in meta
* check if toc is rendered
* get stats at the end of build
* make configurable options: github, website, url, http/https, templates
* remove personal files and replace with samples (logo, ...)
* don't remove tir.yml if it already exists
"""

import logging
import ntpath
import os

from csscompressor import compress

from tir.config import REQUIRED_PATHS

log = logging.getLogger(__name__)


def is_init() -> bool:
    if all([os.path.exists(el) for el in REQUIRED_PATHS]):
        return True
    else:
        log.info('A Tir project already exists at this location.')
        return False


def minify_file(path: str) -> str:
    with open(path, 'r', encoding='utf-8') as f:
        content = f.read()

    css = compress(content)

    with open(path, 'w+') as f:
        f.write(css)

    return ntpath.basename(path)



import os
from typing import List, Set

from tir.posts import Post


def acquire_files(include_private=False) -> Set[Post]:

    posts: Set[Post] = set()

    for root, dirs, files in os.walk(Post.CONTENT_DIR):
        for file in files:
            file_path: str = f'{root.replace(os.getcwd(), "")[1:]}/{file}'
            p: Post = Post(file_path)
            posts.add(p)

    return posts


def rm_dir_files(path: str):
    for n in os.scandir(path):
        os.remove(n.path)


def hash_content(path: str) -> List[int]:
    files = []
    content_hashes = []
    if os.path.isdir(path):
        files.extend([f.path for f in os.scandir(path)])
    elif os.path.isfile(path):
        files.append(path)
    else:
        print('This is a special file. Not handling...')

    for file in files:
        with open(file, 'r', encoding='utf-8') as f:
            content_hashes.append(f.read().__hash__())

    return content_hashes



from dataclasses import dataclass, field
from pathlib import Path
from typing import ClassVar

from tir.schemas.config import Deployment


@dataclass
class Cache:
    DEFAULT_PATH: ClassVar[str] = '.tir_cache'
    path: Path = field(init=False)

    def __post_init__(self):
        self.path = Path.cwd() / self.DEFAULT_PATH
        if not self.path.exists():
            self.path.mkdir()

    def create_deployment_cache(self, config: Deployment) -> Path:
        deployment_cache_path = self.path.joinpath('deployments')
        deployment_cache_path.mkdir(exist_ok=True)
        deployment_cache_folders = [
            x for x in deployment_cache_path.iterdir() if not x.is_file()]

        deployement_id: int = max(
            [int(d.name) for d in deployment_cache_folders]) if deployment_cache_folders else -1
        deployement_id = deployement_id + 1

        deployment_cache_folder_path = deployment_cache_path.joinpath(
            str(deployement_id))
        deployment_cache_folder_path.mkdir()

        return deployment_cache_folder_path



import os
import webbrowser
from http.server import HTTPServer, BaseHTTPRequestHandler

from tir.templates import TemplateLoader


class TirServer(BaseHTTPRequestHandler):
    def _set_headers(self):
        self.tpl = TemplateLoader(layout_directory=os.path.join(os.getcwd(), 'layout', 'default'))
        self.send_response(200)
        self.send_header("Content-type", "text/html")
        self.end_headers()

    def do_GET(self):
        self._set_headers()
        response = ''
        if self.path == '/':
            posts = [file for file in os.scandir(os.path.join(os.getcwd(), 'content', 'posts'))]
            self.wfile.write(self.tpl.env.get_template('server/index.html').render(posts=posts).encode('utf8'))
        elif self.path == '/write':
            self.wfile.write(self.tpl.env.get_template('server/index.html').render().encode('utf8'))

    def do_HEAD(self):
        self._set_headers()

    def do_POST(self):
        # Doesn't do anything with posted data
        self._set_headers()
        self.wfile.write(self._html("POST!"))


def run(server_class=HTTPServer, handler_class=TirServer, addr="localhost", port=8000):
    server_address = (addr, port)
    httpd = server_class(server_address, handler_class)

    webbrowser.open(f'{addr}:{port}')

    print(f"Starting httpd server on {addr}:{port}")

    httpd.serve_forever()



import os

from tir import Post, mktree


def build_notes_index(notes_dir: str):
    print('Building notes...')
    with os.scandir(notes_dir) as it:
        for entry in it:
            if entry.name.endswith('.md') and entry.is_file():
                p = Post(entry.path)
                target_path = '%s/%s%s' % (self.build_dir, p.file_base_name, self.file_ext)
                mktree(self.build_dir)
                with open(target_path, 'w', encoding='utf-8') as fh:
                    head = {'stylesheet_file_name': minified_stylesheet_path}
                    fh.write(tpl_loader.env.get_template(
                        'index.html' if p.file_base_name == 'index' else 'post.html').render(
                        post=p,
                        head=head
                    ))
                print('Compiling {}...'.format(target_path))


import argparse
import logging

from tir import Tir, server

log = logging.getLogger(__name__)


def main():
    parser = argparse.ArgumentParser(description='Tir CLI')
    group = parser.add_argument_group('test')
    group.add_argument('command', help='command to execute',
                       choices=['init', 'build', 'watch', 'server', 'test'])

    args = parser.parse_args()

    tir = Tir()

    if args.command == 'init':
        tir.init()
    elif args.command == 'build':
        tir.build()
    elif args.command == 'watch':
        tir.watch()
    elif args.command == 'server':
        server.run()
    else:
        print('Command not found')


if __name__ == '__main__':
    main()



import xml.etree.ElementTree as etree

from markdown.extensions import Extension
from markdown.inlinepatterns import LinkInlineProcessor

from tir.utils import get_domain


class LocalLinksProcessor(LinkInlineProcessor):
    def handleMatch(self, m, data):
        text, index, handled = self.getText(data, m.end(0))

        if not handled:
            return None, None, None

        href, title, index, handled = self.getLink(data, index)
        print(href)
        if not handled:
            return None, None, None

        el = etree.Element("a")
        el.text = text

        el.set("href", href)

        icons = ['wikipedia']
        site_name = get_domain(href, include_ext=False)

        if site_name in icons:
            el.set('class', 'icon ' + site_name)

        if site_name in ['ouafi', '']:
            el.set('class', 'icon internal')

        if title is not None:
            el.set("title", title)

        return el, m.start(0), index


class LocalLinksExtension(Extension):

    def extendMarkdown(self, md):
        md.inlinePatterns.register(LocalLinksProcessor(r'\[.*?\]', md), 'local_link', 1)



from colorama import Fore


def cprint(color: str, txt: str):
    if color == 'GREEN':
        print(Fore.GREEN + txt)
    elif color == 'RED':
        print(Fore.RED + txt)
    elif color == 'YELLOW':
        print(Fore.YELLOW + txt)
    else:
        print(txt)



import os

import markdown
from jinja2 import Environment, FileSystemLoader
from markdown.extensions.wikilinks import WikiLinkExtension

from tir.parsers.markdown.links import CustomInlineLinksExtension
from tir.parsers.markdown.local_links import LocalLinksExtension
from tir.utils import format_date, url_for


class TemplateLoader:

    def __init__(self, layout_directory: str = None, config=None):
        if config is None:
            config = {}
        self.conf = config
        self.env = Environment(
            loader=FileSystemLoader(os.path.join(
                layout_directory, 'templates')),
            autoescape=True
        )
        self.md = markdown.Markdown(
            extensions=[
                'markdown.extensions.meta',
                'markdown.extensions.toc',
                'markdown.extensions.footnotes',
                'markdown.extensions.def_list',
                'markdown.extensions.tables',
                'markdown.extensions.sane_lists',
                WikiLinkExtension(
                    base_url=f'https://fr.wikipedia.org/wiki/', end_url=''),
                LocalLinksExtension(),
                CustomInlineLinksExtension()
            ]
        )
        self.env.globals['url_for'] = url_for
        self.env.globals['format_date'] = format_date
        self.env.globals['config'] = self.conf
        self.env.globals['parse_md'] = self.md.convert



import os

import pytest

from tests import DATA_DIR, SAMPLE_PAGE_FILE_PATH


@pytest.fixture
def sample_env_file_path():
    return os.path.join(DATA_DIR, '.env-test')


@pytest.fixture
def load_dummy_file_content():
    with open(SAMPLE_PAGE_FILE_PATH, 'r') as f:
        content = f.read()
    return content



import time
from dataclasses import field
from pathlib import Path
from typing import Any, List, Set

from marshmallow import Schema
from marshmallow_dataclass import dataclass


@dataclass
class LockFileItem(Schema):
    path: str
    signature: str


@dataclass
class LockFile(Schema):
    last_deployment: float = field(default=time.time())
    content: List[LockFileItem] = field(default_factory=list)


@dataclass
class StaticFile(Schema):
    content: bytes = None
    path: Any = field(default_factory=Path.cwd)


'''@dataclass
class DeltaLock:
    to_delete: Set[Path]
'''